@@ -5,7 +5,7 @@
  * Massachusetts.
  *
  * Enhancements Copyright 1992-2001, 2002, 2003, 2004, 2005, 2006,
- * 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
+ * 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
  *
  * Enhancements Copyright 2005 Alessandro Scotti
  *
@@ -110,6 +110,8 @@ extern char* programVersion;
 extern ProcRef firstProgramPR, secondProgramPR;
 extern Board boards[];
 extern char marker[BOARD_RANKS][BOARD_FILES];
+extern char lastMsg[MSG_SIZ];
+extern Boolean bookUp;
 
 char *CmailMsg P((void));
 /* Tord: Added the useFEN960 parameter in PositionToFEN() below */
@@ -124,7 +126,7 @@ void SetBlackToPlayEvent P((void));
 void UploadGameEvent P((void));
 void InitBackEnd1 P((void));
 void InitBackEnd2 P((void));
-int HasPromotionChoice P((int fromX, int fromY, int toX, int toY, char *choice));
+int HasPromotionChoice P((int fromX, int fromY, int toX, int toY, char *choice, int sweep));
 int InPalace P((int row, int column));
 int PieceForSquare P((int x, int y));
 int OKToStartUserMove P((int x, int y));
@@ -206,9 +208,11 @@ void ExitAnalyzeMode P((void));
 void AnalyzeModeEvent P((void));
 void AnalyzeFileEvent P((void));
 void MatchEvent P((int mode));
+void RecentEngineEvent P((int nr));
+void TypeInEvent P((char first));
+void TypeInDoneEvent P((char *move));
 void InitPosition P((int redraw));
 void NewSettingEvent P((int option, int *feature, char *command, int value));
-int WaitForSecond P((DelayedEventCallback x));
 void SettingsMenuIfReady P((void));
 void DoEcho P((void));
 void DontEcho P((void));
@@ -229,6 +233,14 @@ Boolean LoadPV P((int x, int y));
 Boolean LoadMultiPV P((int x, int y, char *buf, int index, int *start, int *end));
 void UnLoadPV P(());
 void MovePV P((int x, int y, int h));
+int PromoScroll P((int x, int y));
+void EditBookEvent P((void));
+Boolean DisplayBook P((int moveNr));
+void SaveToBook P((char *text));
+int PackGame P((Board board));
+Boolean ParseFEN P((Board board, int *blackPlaysFirst, char *fen));
+void ApplyMove P((int fromX, int fromY, int toX, int toY, int promoChar, Board board));
+void PackMove P((int fromX, int fromY, int toX, int toY, ChessSquare promoPiece));
 
 char *StrStr P((char *string, char *match));
 char *StrCaseStr P((char *string, char *match));
@@ -272,6 +284,8 @@ char *PGNResult P((ChessMove result));
 typedef struct _ListGame {
     ListNode node;
     int number;
+    int position;
+    int moves;
     unsigned long offset;   /*  Byte offset of game within file.     */
     GameInfo gameInfo;      /*  Note that some entries may be NULL. */
 } ListGame;
@@ -286,8 +300,15 @@ int GameListBuild P((FILE *));
 void GameListInitGameInfo P((GameInfo *));
 char *GameListLine P((int, GameInfo *));
 char * GameListLineFull P(( int, GameInfo *));
+void InitSearch P((void));
+int GameContainsPosition P((FILE *f, ListGame *lg));
 void GLT_TagsToList P(( char * tags ));
 void GLT_ParseList P((void));
+int NamesToList P((char *name, char **engines, char **mnemonics, char *group));
+int CreateTourney P((char *name));
+char *MakeName P((char *templ));
+void SwapEngines P((int n));
+void Substitute P((char *participants, int expunge));
 
 extern char* StripHighlight P((char *));  /* returns static data */
 extern char* StripHighlightAndTitle P((char *));  /* returns static data */
@@ -299,9 +320,17 @@ int Explode P((Board board, int fromX, int fromY, int toX, int toY));
 typedef enum { CheckBox, ComboBox, TextBox, Button, Spin, ResetButton, SaveButton,
 		 FileName, PathName, Slider, Message, Fractional, Label, Break, EndMark } Control;
 
+/* Flags Option.min used for ComboBox: */
+#define COMBO_CALLBACK	(1 << 0)
+#define NO_GETTEXT	(1 << 1)
+
+/* Flags for Option.min used for Button, SaveButton, EndMark: */
+#define SAME_ROW	(1 << 0)
+#define NO_OK		(1 << 1)
+
 typedef struct _OPT {   // [HGM] options: descriptor of UCI-style option
     int value;          // current setting, starts as default
-    int min;
+    int min;		// Also used for flags
     int max;
     void *handle;       // for use by front end
     void *target;       // for use by front end
@@ -330,6 +359,7 @@ typedef struct _CPS {
     int offeredDraw; /* countdown */
     int reuse;
     int useSetboard; /* 0=use "edit"; 1=use "setboard" */
+    int extendedEdit;/* 1=also set holdings with "edit" */
     int useSAN;      /* 0=use coordinate notation; 1=use SAN */
     int usePing;     /* 0=not OK to use ping; 1=OK */
     int lastPing;
@@ -405,6 +435,23 @@ extern Boolean shuffleOpenings;
 extern ChessProgramStats programStats;
 extern int opponentKibitzes; // used by wengineo.c
 extern int errorExitStatus;
+extern char *recentEngines;
+extern char *comboLine;
 void SettingsPopUp P((ChessProgramState *cps)); // [HGM] really in front-end, but CPS not known in frontend.h
+int WaitForEngine P((ChessProgramState *cps, DelayedEventCallback x));
+void Load P((ChessProgramState *cps, int n));
+int MultiPV P((ChessProgramState *cps));
+void MoveHistorySet P(( char movelist[][2*MOVE_LEN], int first, int last, int current, ChessProgramStats_Move * pvInfo ));
+void EvalGraphSet P(( int first, int last, int current, ChessProgramStats_Move * pvInfo ));
+void MakeEngineOutputTitle P((void));
+
+/* A point in time */
+typedef struct {
+    long sec;  /* Assuming this is >= 32 bits */
+    int ms;    /* Assuming this is >= 16 bits */
+} TimeMark;
+
+void GetTimeMark P((TimeMark *));
+long SubtractTimeMarks P((TimeMark *, TimeMark *));
 
 #endif /* _BACKEND */
