@@ -5,6 +5,8 @@
  *
  * Copyright 2005 Alessandro Scotti
  *
+ * Enhancements Copyright 1995, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+ *
  * ------------------------------------------------------------------------
  *
  * GNU XBoard is free software: you can redistribute it and/or modify
@@ -43,7 +45,23 @@
 #include "common.h"
 #include "frontend.h"
 #include "backend.h"
+#include "moves.h"
 #include "engineoutput.h"
+#include "gettext.h"
+
+#ifdef ENABLE_NLS
+# define  _(s) gettext (s)
+# define N_(s) gettext_noop (s)
+#else
+# ifdef WIN32
+#  define  _(s) T_(s)
+#  undef  ngettext
+#  define  ngettext(s,p,n) T_(p)
+# else
+#  define  _(s) (s)
+# endif
+# define N_(s)  s
+#endif
 
 typedef struct {
     char * name;
@@ -70,12 +88,42 @@ static int  lastDepth[2] = { -1, -1 };
 static int  lastForwardMostMove[2] = { -1, -1 };
 static int  engineState[2] = { -1, -1 };
 static char lastLine[2][MSG_SIZ];
+static char header[MSG_SIZ];
 
 #define MAX_VAR 400
 static int scores[MAX_VAR], textEnd[MAX_VAR], curDepth[2], nrVariations[2];
 
+extern int initialRulePlies;
+
+void
+MakeEngineOutputTitle ()
+{
+	static char buf[MSG_SIZ];
+	static char oldTitle[MSG_SIZ];
+	char title[MSG_SIZ];
+	int count, rule = 2*appData.ruleMoves;
+
+	snprintf(title, MSG_SIZ, _("Engine Output") );
+
+	if(!EngineOutputIsUp()) return;
+	// figure out value of 50-move counter
+	count = currentMove;
+	while( (signed char)boards[count][EP_STATUS] <= EP_NONE && count > backwardMostMove ) count--;
+	if( count == backwardMostMove ) count -= initialRulePlies;
+	count = currentMove - count;
+	if(!rule) rule = 100;
+	if(count >= rule - 40 && (!appData.icsActive || gameMode == IcsObserving)) {
+		snprintf(buf, MSG_SIZ, ngettext("%s (%d reversible ply)", "%s (%d reversible plies)", count), title, count);
+		safeStrCpy(title, buf, MSG_SIZ);
+	}
+	if(!strcmp(oldTitle, title)) return;
+	safeStrCpy(oldTitle, title, MSG_SIZ);
+	SetEngineOutputTitle(title);
+}
+
 // back end, due to front-end wrapper for SetWindowText, and new SetIcon arguments
-void SetEngineState( int which, int state, char * state_data )
+void
+SetEngineState (int which, int state, char * state_data)
 {
     int x_which = 1 - which;
 
@@ -107,12 +155,12 @@ void SetEngineState( int which, int state, char * state_data )
 }
 
 // back end, now the front-end wrapper ClearMemo is used, and ed no longer contains handles.
-void SetProgramStats( FrontEndProgramStats * stats ) // now directly called by back-end
+void
+SetProgramStats (FrontEndProgramStats * stats) // now directly called by back-end
 {
     EngineOutputData ed;
     int clearMemo = FALSE;
-    int which;
-    int depth;
+    int which, depth, multi;
 
     if( stats == 0 ) {
         SetEngineState( 0, STATE_IDLE, "" );
@@ -171,6 +219,12 @@ void SetProgramStats( FrontEndProgramStats * stats ) // now directly called by b
 
     if( clearMemo ) {
         DoClearMemo(which); nrVariations[which] = 0;
+        header[0] = NULLCHAR;
+        if(gameMode == AnalyzeMode && (multi = MultiPV(&first)) >= 0) {
+            snprintf(header, MSG_SIZ, "\t%s viewpoint\t\tfewer / Multi-PV setting = %d / more\n",
+                                       appData.whitePOV || appData.scoreWhite ? "white" : "mover", first.option[multi].value);
+            InsertIntoMemo( which, header, 0);
+        } else
         if(appData.ponderNextMove && lastLine[which][0]) {
             InsertIntoMemo( which, lastLine[which], 0 );
             InsertIntoMemo( which, "\n", 0 );
@@ -189,7 +243,8 @@ void SetProgramStats( FrontEndProgramStats * stats ) // now directly called by b
 #define ENGINE_COLOR_UNKNOWN    ' '
 
 // pure back end
-static char GetEngineColor( int which )
+static char
+GetEngineColor (int which)
 {
     char result = ENGINE_COLOR_UNKNOWN;
 
@@ -222,7 +277,8 @@ static char GetEngineColor( int which )
 }
 
 // pure back end
-static char GetActiveEngineColor()
+static char
+GetActiveEngineColor ()
 {
     char result = ENGINE_COLOR_UNKNOWN;
 
@@ -234,7 +290,8 @@ static char GetActiveEngineColor()
 }
 
 // pure back end
-static int IsEnginePondering( int which )
+static int
+IsEnginePondering (int which)
 {
     int result = FALSE;
 
@@ -259,7 +316,8 @@ static int IsEnginePondering( int which )
 }
 
 // back end
-static void SetDisplayMode( int mode )
+static void
+SetDisplayMode (int mode)
 {
     if( windowMode != mode ) {
         windowMode = mode;
@@ -269,7 +327,8 @@ static void SetDisplayMode( int mode )
 }
 
 // pure back end
-static void VerifyDisplayMode()
+static void
+VerifyDisplayMode ()
 {
     int mode;
 
@@ -299,7 +358,8 @@ static void VerifyDisplayMode()
 }
 
 // back end. Determine what icon to set in the color-icon field, and print it
-void SetEngineColorIcon( int which )
+void
+SetEngineColorIcon (int which)
 {
     char color = GetEngineColor(which);
     int nicon = 0;
@@ -318,7 +378,8 @@ void SetEngineColorIcon( int which )
 
 // [HGM] multivar: sort Thinking Output within one depth on score
 
-static int InsertionPoint( int len, EngineOutputData * ed )
+static int
+InsertionPoint (int len, EngineOutputData *ed)
 {
 	int i, offs = 0, newScore = ed->score, n = ed->which;
 
@@ -345,17 +406,18 @@ static int InsertionPoint( int len, EngineOutputData * ed )
 		scores[n] = newScore;
 	}
 	nrVariations[n] += 2;
-      return offs;
+      return offs + (gameMode == AnalyzeMode)*strlen(header);
 }
 
 
 // pure back end, now SetWindowText is called via wrapper DoSetWindowText
-static void UpdateControls( EngineOutputData * ed )
+static void
+UpdateControls (EngineOutputData *ed)
 {
 //    int isPondering = FALSE;
 
     char s_label[MAX_NAME_LENGTH + 32];
-
+    int h;
     char * name = ed->name;
 
     /* Label */
@@ -455,11 +517,12 @@ static void UpdateControls( EngineOutputData * ed )
         }
 
         /* Score */
-        if( ed->score > 0 ) {
-	  snprintf( s_score, sizeof(s_score)/sizeof(s_score[0]), "+%.2f", ed->score / 100.0 );
+        h = ((gameMode == AnalyzeMode && appData.whitePOV || appData.scoreWhite) && !WhiteOnMove(currentMove) ? -1 : 1) * ed->score;
+        if( h > 0 ) {
+	  snprintf( s_score, sizeof(s_score)/sizeof(s_score[0]), "+%.2f", h / 100.0 );
         }
         else {
-	  snprintf( s_score, sizeof(s_score)/sizeof(s_score[0]), "%.2f", ed->score / 100.0 );
+	  snprintf( s_score, sizeof(s_score)/sizeof(s_score[0]), "%.2f", h / 100.0 );
         }
 
         /* Time */
@@ -490,7 +553,8 @@ static void UpdateControls( EngineOutputData * ed )
 }
 
 // [HGM] kibitz: write kibitz line; split window for it if necessary
-void OutputKibitz(int window, char *text)
+void
+OutputKibitz (int window, char *text)
 {
 	static int currentLineEnd[2];
 	int where = 0;
