@@ -5,7 +5,7 @@
  * Massachusetts.
  *
  * Enhancements Copyright 1992-2001, 2002, 2003, 2004, 2005, 2006,
- * 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
+ * 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
  *
  * Enhancements Copyright 2005 Alessandro Scotti
  *
@@ -83,6 +83,8 @@ void DisplayNote P((String message));
 void AskQuestion P((String title, String question, String replyPrefix,
 		    ProcRef pr));
 void DisplayIcsInteractionTitle P((String title));
+void ParseArgsFromString P((char *p));
+void ParseArgsFromFile P((FILE *f));
 void DrawPosition P((int fullRedraw, Board board));
 void ResetFrontEnd P((void));
 void NotifyFrontendLogin P((void));
@@ -95,6 +97,7 @@ void DrawSeekAxis P(( int x, int y, int xTo, int yTo ));
 void DrawSeekBackground P(( int left, int top, int right, int bottom ));
 void DrawSeekText P((char *buf, int x, int y));
 void DrawSeekDot P((int x, int y, int color));
+void PopUpMoveDialog P((char first));
 
 void RingBell P((void));
 void PlayIcsWinSound P((void));
@@ -102,10 +105,13 @@ void PlayIcsLossSound P((void));
 void PlayIcsDrawSound P((void));
 void PlayIcsUnfinishedSound P((void));
 void PlayAlarmSound P((void));
+void PlayTellSound P((void));
 void EchoOn P((void));
 void EchoOff P((void));
 void Raw P((void));
 void Colorize P((ColorClass cc, int continuation));
+char *InterpretFileName P((char *name, char *dir));
+void DoSleep P((int n));
 
 char *UserName P((void));
 char *HostName P((void));
@@ -115,6 +121,7 @@ int StopClockTimer P((void));
 void StartClockTimer P((long millisec));
 void DisplayWhiteClock P((long timeRemaining, int highlight));
 void DisplayBlackClock P((long timeRemaining, int highlight));
+void UpdateLogos P((int display));
 
 int LoadGameTimerRunning P((void));
 int StopLoadGameTimer P((void));
@@ -129,7 +136,7 @@ void CancelDelayedEvent P((void));
 extern int fromX, fromY, toX, toY;
 typedef enum { Press, Release } ClickType;
 void PromotionPopUp P((void));
-void DragPieceBegin P((int x, int y));
+void DragPieceBegin P((int x, int y, Boolean instantly));
 void DragPieceEnd P((int x, int y));
 void LeftClick P((ClickType c, int x, int y));
 int  RightClick P((ClickType c, int x, int y, int *col, int *row));
@@ -137,6 +144,7 @@ int  RightClick P((ClickType c, int x, int y, int *col, int *row));
 int StartChildProcess P((char *cmdLine, char *dir, ProcRef *pr));
 void DestroyChildProcess P((ProcRef pr, int/*boolean*/ signal));
 void InterruptChildProcess P((ProcRef pr));
+void RunCommand P((char *buf));
 
 int OpenTelnet P((char *host, char *port, ProcRef *pr));
 int OpenTCP P((char *host, char *port, ProcRef *pr));
@@ -169,6 +177,7 @@ void GLT_AddToList( char *name );
 Boolean GLT_GetFromList( int index, char *name );
 
 extern char lpUserGLT[];
+extern char *homeDir;
 
 /* these are in wgamelist.c */
 void GameListPopUp P((FILE *fp, char *filename));
@@ -198,8 +207,11 @@ void AnimateMove P((Board board, int fromX, int fromY, int toX, int toY));
 void HistorySet P((char movelist[][2*MOVE_LEN], int first, int last, int current));
 void FreezeUI P((void));
 void ThawUI P((void));
+void ChangeDragPiece P((ChessSquare piece));
+void CopyFENToClipboard P((void));
 extern char *programName;
 extern int commentUp;
+extern char *firstChessProgramNames;
 
 void GreyRevert P((Boolean grey));
 
