@@ -5,7 +5,7 @@
  * Massachusetts.
  *
  * Enhancements Copyright 1992-2001, 2002, 2003, 2004, 2005, 2006,
- * 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
+ * 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
  *
  * Enhancements Copyright 2005 Alessandro Scotti
  *
@@ -185,6 +185,7 @@
 #define EDITTAGS_FONT_TITANIC  "Courier New:8.0"
 
 #define MOVEHISTORY_FONT_ALL    "MS Sans Serif:8.0"
+#define GAMELIST_FONT_ALL       "MS Sans Serif:8.0"
 
 #define COLOR_SHOUT            "#209000"
 #define COLOR_SSHOUT         "b #289808"
