@@ -1,7 +1,7 @@
 /*
  * xgamelist.c -- Game list window, part of X front end for XBoard
  *
- * Copyright 1995, 2009, 2010, 2011 Free Software Foundation, Inc.
+ * Copyright 1995, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
  * ------------------------------------------------------------------------
  *
  * GNU XBoard is free software: you can redistribute it and/or modify
@@ -95,12 +95,14 @@ void SetFocus P((Widget w, XtPointer data, XEvent *event, Boolean *b));
 
 static Widget filterText;
 static char filterString[MSG_SIZ];
-static int listLength, page;
+static int listLength, wins, losses, draws, page;
 
 char gameListTranslations[] =
   "<Btn1Up>(2): LoadSelectedProc(0) \n \
    <Key>Home: LoadSelectedProc(-2) \n \
    <Key>End: LoadSelectedProc(2) \n \
+   Ctrl<Key>Up: LoadSelectedProc(-3) \n \
+   Ctrl<Key>Down: LoadSelectedProc(3) \n \
    <Key>Up: LoadSelectedProc(-1) \n \
    <Key>Down: LoadSelectedProc(1) \n \
    <Key>Left: LoadSelectedProc(-1) \n \
@@ -123,10 +125,7 @@ typedef struct {
 static GameListClosure *glc = NULL;
 
 Widget
-GameListCreate(name, callback, client_data)
-     char *name;
-     XtCallbackProc callback;
-     XtPointer client_data;
+GameListCreate (char *name, XtCallbackProc callback, XtPointer client_data)
 {
     Arg args[16];
     Widget shell, form, viewport, listwidg, layout, label;
@@ -171,6 +170,7 @@ GameListCreate(name, callback, client_data)
       XtCreateManagedWidget("viewport", viewportWidgetClass, form, args, j);
 
     j = 0;
+//    XtSetArg(args[j], XtNlist, glc->strings);  j++;
     XtSetArg(args[j], XtNdefaultColumns, 1);  j++;
     XtSetArg(args[j], XtNforceColumns, True);  j++;
     XtSetArg(args[j], XtNverticalList, True);  j++;
@@ -187,7 +187,7 @@ GameListCreate(name, callback, client_data)
     XtSetArg(args[j], XtNleft, XtChainLeft); j++;
     XtSetArg(args[j], XtNright, XtChainLeft); j++;
     b_load =
-      XtCreateManagedWidget(_("load"), commandWidgetClass, form, args, j);
+      XtCreateManagedWidget(_("thresholds"), commandWidgetClass, form, args, j);
     XtAddCallback(b_load, XtNcallback, callback, client_data);
 
     j = 0;
@@ -198,9 +198,9 @@ GameListCreate(name, callback, client_data)
     XtSetArg(args[j], XtNleft, XtChainLeft); j++;
     XtSetArg(args[j], XtNright, XtChainLeft); j++;
     b_loadprev =
-      XtCreateManagedWidget(_("prev"), commandWidgetClass, form, args, j);
+      XtCreateManagedWidget(_("find position"), commandWidgetClass, form, args, j);
     XtAddCallback(b_loadprev, XtNcallback, callback, client_data);
-
+#if 1
     j = 0;
     XtSetArg(args[j], XtNfromVert, viewport);  j++;
     XtSetArg(args[j], XtNfromHoriz, b_loadprev);  j++;
@@ -211,7 +211,9 @@ GameListCreate(name, callback, client_data)
     b_loadnext =
       XtCreateManagedWidget(_("next"), commandWidgetClass, form, args, j);
     XtAddCallback(b_loadnext, XtNcallback, callback, client_data);
-
+#else
+    b_loadnext = b_loadprev;
+#endif
     j = 0;
     XtSetArg(args[j], XtNfromVert, viewport);  j++;
     XtSetArg(args[j], XtNfromHoriz, b_loadnext);  j++;
@@ -241,7 +243,7 @@ GameListCreate(name, callback, client_data)
     XtSetArg(args[j], XtNbottom, XtChainBottom); j++;
     XtSetArg(args[j], XtNleft, XtChainLeft); j++;
     XtSetArg(args[j], XtNright, XtChainRight); j++;
-    XtSetArg(args[j], XtNwidth, fw_width - 225 - squareSize); j++;
+    XtSetArg(args[j], XtNwidth, fw_width - 275 - squareSize); j++;
     XtSetArg(args[j], XtNstring, filterString);  j++;
     XtSetArg(args[j], XtNdisplayCaret, False);  j++;
     XtSetArg(args[j], XtNresizable, True);  j++;
@@ -321,53 +323,79 @@ GameListCreate(name, callback, client_data)
     return shell;
 }
 
+extern int soughtCounts[];
+extern Board soughtBoard;
+
 static int
-GameListPrepare()
+GameListPrepare (int byPos)
 {   // [HGM] filter: put in separate routine, to make callable from call-back
     int nstrings;
     ListGame *lg;
     char **st, *line;
+    TimeMark t, t2;
 
+    GetTimeMark(&t);
+    if(st = glc->strings) while(*st) free(*st++);
     nstrings = ((ListGame *) gameList.tailPred)->number;
     glc->strings = (char **) malloc((nstrings + 1) * sizeof(char *));
     st = glc->strings;
     lg = (ListGame *) gameList.head;
-    listLength = 0;
+    listLength = wins = losses = draws = 0;
+    if(byPos) InitSearch();
     while (nstrings--) {
+	int pos = -1;
 	line = GameListLine(lg->number, &lg->gameInfo);
-	if(filterString[0] == NULLCHAR || SearchPattern( line, filterString ) ) {
-	    *st++ = line; // [HGM] filter: make adding line conditional
+	if((filterString[0] == NULLCHAR || SearchPattern( line, filterString )) && (!byPos || (pos=GameContainsPosition(glc->fp, lg)) >= 0) ) {
+            *st++ = line; // [HGM] filter: make adding line conditional.
 	    listLength++;
+            if( lg->gameInfo.result == WhiteWins ) wins++; else
+            if( lg->gameInfo.result == BlackWins ) losses++; else
+            if( lg->gameInfo.result == GameIsDrawn ) draws++;
 	}
+	if(lg->number % 2000 == 0) {
+	    char buf[MSG_SIZ];
+	    snprintf(buf, MSG_SIZ, _("Scanning through games (%d)"), lg->number);
+	    DisplayTitle(buf);
+	}
+	lg->position = pos;
 	lg = (ListGame *) lg->node.succ;
      }
+GetTimeMark(&t2);printf("GameListPrepare %ld msec\n", SubtractTimeMarks(&t2,&t));
+     DisplayTitle("XBoard");
     *st = NULL;
     return listLength;
 }
 
 static char *list[1003];
+int listEnd;
 
 static void
-GameListReplace(int page)
+GameListReplace (int page)
 {
   // filter: put in separate routine, to make callable from call-back
   Widget listwidg;
-  char **st=list;
+  Arg arg;
+  char buf[MSG_SIZ], *p, **st=list;
   int i;
 
   if(page) *st++ = _("previous page"); else if(listLength > 1000) *st++ = "";
-  for(i=0; i<1000; i++) if( !(*st++ = glc->strings[page+i]) ) break;
+  for(i=0; i<1000; i++) if( !(*st++ = glc->strings[page+i]) ) { st--; break; }
+  listEnd = st - list;
   if(page + 1000 <= listLength) *st++ = _("next page");
   *st = NULL;
+
   listwidg = XtNameToWidget(glc->shell, "*form.viewport.list");
-  XawListChange(listwidg, listLength ? list : dummyList, 0, 0, True);
+  XtSetArg(arg, XtNlist, listLength ? list : dummyList); // empty list displays message
+  XawListChange(listwidg, list, 0, 0, True);
+  XtSetValues(listwidg, &arg, 1);
   XawListHighlight(listwidg, 0);
+  snprintf(buf, MSG_SIZ, _("%s - %d/%d games (%d-%d-%d)"), glc->filename, listLength, ((ListGame *) gameList.tailPred)->number, wins, losses, draws);
+  XtSetArg(arg, XtNtitle, buf);
+  XtSetValues(glc->shell, &arg, 1);
 }
 
 void
-GameListCallback(w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
+GameListCallback (Widget w, XtPointer client_data, XtPointer call_data)
 {
     String name;
     Arg args[16];
@@ -385,6 +413,10 @@ GameListCallback(w, client_data, call_data)
 	GameListPopDown();
 	return;
     }
+    if (strcmp(name, _("thresholds")) == 0) {
+	LoadOptionsProc();
+	return;
+    }
     listwidg = XtNameToWidget(glc->shell, "*form.viewport.list");
     rs = XawListShowCurrent(listwidg);
     if (strcmp(name, _("load")) == 0) {
@@ -407,14 +439,15 @@ GameListCallback(w, client_data, call_data)
 	    return;
 	}
 	XawListHighlight(listwidg, index);
-    } else if (strcmp(name, _("apply")) == 0) {
-        String name;
+    } else if (strcmp(name, _("apply")) == 0 ||
+               strcmp(name, _("find position")) == 0) {
+        String text;
         j = 0;
-        XtSetArg(args[j], XtNstring, &name);  j++;
+        XtSetArg(args[j], XtNstring, &text);  j++;
 	XtGetValues(filterText, args, j);
-        safeStrCpy(filterString, name, sizeof(filterString)/sizeof(filterString[0]));
+        safeStrCpy(filterString, text, sizeof(filterString)/sizeof(filterString[0]));
 	XawListHighlight(listwidg, 0);
-        GameListPrepare(); GameListReplace(0);
+        GameListPrepare(strcmp(name, _("find position")) == 0); GameListReplace(0);
         return;
     }
 #if 1
@@ -430,9 +463,7 @@ GameListCallback(w, client_data, call_data)
 }
 
 void
-GameListPopUp(fp, filename)
-     FILE *fp;
-     char *filename;
+GameListPopUp (FILE *fp, char *filename)
 {
     Arg args[16];
     int j;
@@ -441,35 +472,27 @@ GameListPopUp(fp, filename)
     if (glc == NULL) {
 	glc = (GameListClosure *) calloc(1, sizeof(GameListClosure));
 	glc->x = glc->y = -1;
+	glc->filename = NULL;
+	glc->shell = NULL;
     }
 
-    if (glc->strings != NULL) {
-	st = glc->strings;
-	while (*st) {
-	    free(*st++);
-	}
-	free(glc->strings);
-    }
-
-    GameListPrepare(); // [HGM] filter: code put in separate routine
+    GameListPrepare(False); // [HGM] filter: code put in separate routine
 
     glc->fp = fp;
 
     if (glc->filename != NULL) free(glc->filename);
     glc->filename = StrSave(filename);
 
-
     if (glc->shell == NULL) {
 	glc->shell = GameListCreate(filename, GameListCallback, glc);
     } else {
 	j = 0;
 	XtSetArg(args[j], XtNiconName, (XtArgVal) filename);  j++;
-	XtSetArg(args[j], XtNtitle, (XtArgVal) filename);  j++;
+//	XtSetArg(args[j], XtNtitle, (XtArgVal) filename);  j++;
 	XtSetValues(glc->shell, args, j);
     }
-
     page = 0;
-    GameListReplace(0); // [HGM] filter: code put in separate routine
+    GameListReplace(0); // [HGM] filter: code put in separate routine, and also called to set title
 
     XtPopup(glc->shell, XtGrabNone);
     glc->up = True;
@@ -480,7 +503,7 @@ GameListPopUp(fp, filename)
 }
 
 void
-GameListDestroy()
+GameListDestroy ()
 {
     if (glc == NULL) return;
     GameListPopDown();
@@ -497,11 +520,7 @@ GameListDestroy()
 }
 
 void
-ShowGameListProc(w, event, prms, nprms)
-     Widget w;
-     XEvent *event;
-     String *prms;
-     Cardinal *nprms;
+ShowGameListProc (Widget w, XEvent *event, String *prms, Cardinal *nprms)
 {
     Arg args[16];
     int j;
@@ -524,17 +543,13 @@ ShowGameListProc(w, event, prms, nprms)
 }
 
 void
-LoadSelectedProc(w, event, prms, nprms)
-     Widget w;
-     XEvent *event;
-     String *prms;
-     Cardinal *nprms;
+LoadSelectedProc (Widget w, XEvent *event, String *prms, Cardinal *nprms)
 {
     Widget listwidg;
     XawListReturnStruct *rs;
     int index, direction = atoi(prms[0]);
 
-    if (glc == NULL) return;
+    if (glc == NULL || listLength == 0) return;
     listwidg = XtNameToWidget(glc->shell, "*form.viewport.list");
     rs = XawListShowCurrent(listwidg);
     index = rs->list_index;
@@ -543,37 +558,36 @@ LoadSelectedProc(w, event, prms, nprms)
         page -= 1000;
         if(page < 0) page = 0; // safety
         GameListReplace(page);
-       return;
+	return;
     }
     if(index == 1001) {
         page += 1000;
         GameListReplace(page);
-       return;
+	return;
     }
 
     if(direction != 0) {
+	int doLoad = abs(direction) > 2;
+	if(doLoad) direction /= 3;
 	index += direction;
 	if(direction == -2) index = 0;
-	if(direction == 2) index = listLength-1;
-	if(index < 0 || index >= listLength) return;
+	if(direction == 2) index = listEnd-1;
+	if(index < 0 || index >= listEnd) return;
 	XawListHighlight(listwidg, index);
-	return;
+	if(!doLoad) return;
     }
-    if(!list[index]) return;
     index = atoi(list[index])-1; // [HGM] filter: read true index from sequence nr of line
     if (cmailMsgLoaded) {
 	CmailLoadGame(glc->fp, index + 1, glc->filename, True);
     } else {
 	LoadGame(glc->fp, index + 1, glc->filename, True);
+	XSync(xDisplay, False);
+	XSetInputFocus(xDisplay, XtWindow(boardWidget), RevertToPointerRoot, CurrentTime);
     }
 }
 
 void
-SetFilterProc(w, event, prms, nprms)
-     Widget w;
-     XEvent *event;
-     String *prms;
-     Cardinal *nprms;
+SetFilterProc (Widget w, XEvent *event, String *prms, Cardinal *nprms)
 {
 	Arg args[16];
         String name;
@@ -582,7 +596,7 @@ SetFilterProc(w, event, prms, nprms)
         XtSetArg(args[j], XtNstring, &name);  j++;
 	XtGetValues(filterText, args, j);
         safeStrCpy(filterString, name, sizeof(filterString)/sizeof(filterString[0]));
-        GameListPrepare(); GameListReplace(0);
+        GameListPrepare(False); GameListReplace(0);
 	list = XtNameToWidget(glc->shell, "*form.viewport.list");
 	XawListHighlight(list, 0);
         j = 0;
@@ -592,7 +606,7 @@ SetFilterProc(w, event, prms, nprms)
 }
 
 void
-GameListPopDown()
+GameListPopDown ()
 {
     Arg args[16];
     int j;
@@ -618,8 +632,7 @@ GameListPopDown()
 }
 
 void
-GameListHighlight(index)
-     int index;
+GameListHighlight (int index)
 {
     Widget listwidg;
     int i=0; char **st;
@@ -631,11 +644,41 @@ GameListHighlight(index)
 }
 
 Boolean
-GameListIsUp()
+GameListIsUp ()
 {
     return glc && glc->up;
 }
 
+int
+SaveGameListAsText (FILE *f)
+{
+    ListGame * lg = (ListGame *) gameList.head;
+    int nItem;
+
+    if( !glc || ((ListGame *) gameList.tailPred)->number <= 0 ) {
+      DisplayError(_("Game list not loaded or empty"), 0);
+        return False;
+    }
+
+    /* Copy the list into the global memory block */
+    if( f != NULL ) {
+ 
+        lg = (ListGame *) gameList.head;
+
+        for (nItem = 0; nItem < ((ListGame *) gameList.tailPred)->number; nItem++){
+            char * st = GameListLineFull(lg->number, &lg->gameInfo);
+	    char *line = GameListLine(lg->number, &lg->gameInfo);
+	    if(filterString[0] == NULLCHAR || SearchPattern( line, filterString ) )
+	            fprintf( f, "%s\n", st );
+	    free(st); free(line);
+            lg = (ListGame *) lg->node.succ;
+        }
+
+        fclose(f);
+	return True;
+    }
+    return False;
+}
 //--------------------------------- Game-List options dialog ------------------------------------------
 
 Widget gameListOptShell, listwidg;
@@ -643,32 +686,36 @@ Widget gameListOptShell, listwidg;
 char *strings[20];
 int stringPtr;
 
-void GLT_ClearList()
+void
+GLT_ClearList ()
 {
     strings[0] = NULL;
     stringPtr = 0;
 }
 
-void GLT_AddToList(char *name)
+void
+GLT_AddToList (char *name)
 {
     strings[stringPtr++] = name;
     strings[stringPtr] = NULL;
 }
 
-Boolean GLT_GetFromList(int index, char *name)
+Boolean
+GLT_GetFromList (int index, char *name)
 {
   safeStrCpy(name, strings[index], MSG_SIZ);
   return TRUE;
 }
 
-void GLT_DeSelectList()
+void
+GLT_DeSelectList ()
 {
     XawListChange(listwidg, strings, 0, 0, True);
     XawListHighlight(listwidg, 0);
 }
 
 void
-GameListOptionsPopDown()
+GameListOptionsPopDown ()
 {
   if (gameListOptShell == NULL) return;
 
@@ -679,9 +726,7 @@ GameListOptionsPopDown()
 }
 
 void
-GameListOptionsCallback(w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
+GameListOptionsCallback (Widget w, XtPointer client_data, XtPointer call_data)
 {
     String name;
     Arg args[16];
@@ -732,7 +777,7 @@ GameListOptionsCallback(w, client_data, call_data)
 }
 
 Widget
-GameListOptionsCreate()
+GameListOptionsCreate ()
 {
     Arg args[16];
     Widget shell, form, viewport, layout;
@@ -749,7 +794,7 @@ GameListOptionsCreate()
     XtSetArg(args[j], XtNresizable, True);  j++;
     XtSetArg(args[j], XtNallowShellResize, True);  j++;
     shell = gameListOptShell =
-      XtCreatePopupShell("Game-list options", transientShellWidgetClass,
+      XtCreatePopupShell(_("Game-list options"), transientShellWidgetClass,
 			 shellWidget, args, j);
     layout =
       XtCreateManagedWidget(layoutName, formWidgetClass, shell,
@@ -833,7 +878,7 @@ GameListOptionsCreate()
 }
 
 void
-GameListOptionsPopUp(Widget w, XEvent *event, String *prms, Cardinal *nprms)
+GameListOptionsPopUp (Widget w, XEvent *event, String *prms, Cardinal *nprms)
 {
   if (gameListOptShell == NULL)
     gameListOptShell = GameListOptionsCreate();
